=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2010-06-30 08:31:48 +0000
+++ bzrlib/branch.py	2010-06-30 08:34:11 +0000
@@ -815,11 +815,14 @@
                 raise AssertionError("didn't expect %r to have "
                     "fallback_repositories"
                     % (self.repository,))
-            # Do our best to transfer the lock state  (i.e. lock-tokens and
+            # Replace self.repository with the new repository.
+            # Do our best to transfer the lock state (i.e. lock-tokens and
             # lock count) of self.repository to the new repository.
             lock_token = old_repository.lock_write().repository_token
             self.repository = new_repository
             if isinstance(self, remote.RemoteBranch):
+                # Remote branches can have a second reference to the old
+                # repository that need to be replaced.
                 if self._real_branch is not None:
                     self._real_branch.repository = new_repository
             self.repository.lock_write(token=lock_token)
@@ -829,7 +832,8 @@
             if lock_token is not None:
                 # XXX: self.repository.leave_lock_in_place() before this
                 # function will not be preserved.  Fortunately that doesn't
-                # affect the current default format (2a).
+                # affect the current default format (2a), and would be a
+                # corner-case anyway.
                 #  - Andrew Bennetts, 2010/06/30
                 self.repository.dont_leave_lock_in_place()
             old_lock_count = 0

=== modified file 'bzrlib/tests/per_branch/test_stacking.py'
--- bzrlib/tests/per_branch/test_stacking.py	2010-06-30 08:25:28 +0000
+++ bzrlib/tests/per_branch/test_stacking.py	2010-06-30 08:34:11 +0000
@@ -207,7 +207,7 @@
         self.assertRaises(errors.NotStacked,
             new_branch.get_stacked_on_url)
 
-    def test_unstack_locking_robustness(self):
+    def test_unstack_already_locked(self):
         """Removing the stacked-on branch with an already write-locked branch
         works.
 
@@ -222,7 +222,12 @@
         stacked_branch.set_stacked_on_url(None)
         stacked_branch.unlock()
 
-    def test_unstack_locking_lots(self):
+    def test_unstack_already_multiple_locked(self):
+        """Unstacking a branch preserves the lock count (even though it
+        replaces the br.repository object).
+
+        This is a more extreme variation of test_unstack_already_locked.
+        """
         try:
             stacked_bzrdir = self.make_stacked_bzrdir()
         except unstackable_format_errors, e:

